unit QRXUtil;

{$IfDef Win32}
{$H-}
{$EndIf}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, FileCtrl, ExtCtrls, DB, DBIProcs, DBITypes;

type
  TDatabaseList = class(TDBDataSet)
  protected
    function CreateHandle: HDBICur; override;
  end;
  
  TAliasComboBox = class(TDriveComboBox)
  private
    FDBList : TDatabaseList;
    FAliasList : TStringList;
    FAliasBitMap : TBitMap;
    FShowingAlias : boolean;
  protected
    procedure Click; override;
  public
    constructor Create(aOwner:TComponent); override;
    destructor Destroy; override;
    procedure BuildAliasList;
    function GetDirectory(AliasName:string):string;
    property ShowingAlias:boolean read FShowingAlias write FShowingAlias;
    property AliasList:TStringList read FAliasList write FAliasList;
    property AliasBitMap:TBitMap read FAliasBitMap write FAliasBitMap;
  end;

  TTableListBox = class(TFileListBox)
  private
    FTableList : TStringList;
    FSystemTables : boolean;
    FTableBitMap : TBitMap;
  protected
    procedure ReadBitmaps; override;
  public
    constructor Create(aOwner:TComponent); override;
    destructor Destroy; override;
    procedure BuildTableList(const AliasName,Pattern:string);
    property SystemTables:boolean read FSystemTables write FSystemTables;
  end;

procedure Register;

function RemoveExt(const FileName:string):string;

function ConstStr(C:char; N:byte):string;

function FirstUpCase(const S:string):string;

function Min(X, Y: Integer): Integer;

function Max(X, Y: Integer): Integer;

implementation

procedure Register;
begin
  RegisterComponents('Samples',[TAliasComboBox,TTableListBox]);
end;

constructor TAliasComboBox.Create(aOwner:TComponent);
begin
  inherited Create(aOwner);
  FDBList := TDatabaseList.Create(Self);
  FDBList.Open;
  FAliasList := TStringList.Create;
  FAliasBitMap := TBitMap.Create;
  FAliasBitMap.Handle := LoadBitMap(HInstance,'QRX_ALIAS');
  FShowingAlias := FALSE;
end;

destructor TAliasComboBox.Destroy;
begin
  FDBList.Free;
  FAliasList.Free;
  FAliasBitMap.Free;
  inherited Destroy;
end;

procedure TAliasComboBox.BuildAliasList;
var
  F : integer;
  SaveDrive : char;
  L : TStringList;
begin
  L := TStringList.Create;
  try
    SaveDrive := Drive;
    Session.GetAliasNames(L);
    for F := 0 to L.Count-1 do
      L.Objects[F] := FAliasBitMap;
    FAliasList.Assign(L);
    L.AddStrings(Items);
    Items.Assign(L);
    Drive := SaveDrive;
  finally
    L.Free;
  end;
end;

function TAliasComboBox.GetDirectory(AliasName:string):string;
begin
  FDBList.First;
  while not FDBList.EOF and (FDBList.FieldByName('NAME').AsString <> AliasName) do
    FDBList.Next;
  if not FDBList.EOF then begin
    Result := FDBList.FieldByName('PHYNAME').AsString;
    if Result[Ord(Result[0])] = '\' then
      Dec(Result[0]);
  end else
    Result := '';
{$IfDef Win32}
  Result := UpperCase(Result);
{$EndIf}
end;

procedure TAliasComboBox.Click;
var
  I : integer;
begin
  I := FAliasList.IndexOf(Text);
  if I <> -1 then begin { using alias }
    FShowingAlias := TRUE;
    Change;
  end else begin
    FShowingAlias := FALSE;
    Change;
  end;
end;

constructor TTableListBox.Create(aOwner:TComponent);
begin
  inherited Create(aOwner);
  FTableBitMap := TBitMap.Create;
  FTableBitMap.Handle := LoadBitMap(HInstance,'QRX_TABLE');
end;

destructor TTableListBox.Destroy;
begin
  FTableBitMap.Free;
  inherited Destroy;
end;

procedure TTableListBox.ReadBitMaps;
begin
  inherited ReadBitMaps;
  UnknownBMP.Handle := LoadBitMap(HInstance,'QRX_TABLE');
end;

procedure TTableListBox.BuildTableList(const AliasName,Pattern:string);
var
  F : integer;
  L : TStringList;
begin
  L := TStringList.Create;
  try
    Session.GetTableNames(AliasName,Pattern,TRUE,FSystemTables,L);
    for F := 0 to L.Count - 1 do
      L.Objects[F] := FTableBitMap;
    Items.Assign(L);
  finally
    L.Free;
  end;
end;

function TDatabaseList.CreateHandle: HDBICur;
begin
  Check(DbiOpenDatabaseList(Result));
end;

function RemoveExt(const FileName:string):string;
var
  P : byte;
begin
  Result := FileName;
  P := Pos('.',Result);
  if P <> 0 then
    System.Delete(Result,P,256);
end;

function ConstStr(C:char; N:byte):string;
begin
  FillChar(Result[1],N,C);
  Result[0] := Char(N);
end;

function FirstUpCase(const S:string):string;
begin
  Result := S;
  Result[1] := UpCase(Result[1]);
end;

function Min(X, Y: Integer): Integer; assembler;
asm
{$IfDef Win32}
	MOV	EAX,X
	CMP	EAX,Y
	JLE	@@1
	MOV	EAX,Y
{$Else}
	MOV	AX,X
	CMP	AX,Y
	JLE	@@1
	MOV	AX,Y
{$EndIf}
@@1:
end;

function Max(X, Y: Integer): Integer; assembler;
asm
{$IfDef Win32}
	MOV	EAX,X
	CMP	EAX,Y
	JGE	@@1
	MOV	EAX,Y
{$Else}
	MOV	AX,X
	CMP	AX,Y
	JGE	@@1
	MOV	AX,Y
{$EndIf}
@@1:
end;

end.
